/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "ExternalVideoPlayer.h"
#include "utility.h"
#include "debug.h"
#include "SubtitleSystem.h"
#include "DocumentSystem.h"

/*
 *
 */
bool ExternalVideoPlayer::execute(const Glib::ustring &uri)
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	g_return_val_if_fail(doc, false);

	if(uri.empty())
	{
		doc->flash_message(_("I can't play movie. Please select a movie."));
		return false;
	}

	// subtitle.[format]
	Glib::ustring tmp_subtitle_name = "subtitle." + SubtitleSystem::getInstance().get_extension(doc->getFormat());

	// tmp dir + subtitle name
	Glib::ustring subtitle_file = Glib::build_filename(Glib::get_tmp_dir(), tmp_subtitle_name);

	// save now tmp subtitle
	
	//doc->save(subtitle_file, doc->getFormat(), "", false);
	Glib::ustring old_filename = doc->getFilename();
	doc->save(subtitle_file);
	doc->setFilename(old_filename);
	
	long start_position = 0;

	SubtitleModifier sub = doc->get_subtitle_view()->getSelected();
	if(sub)
	{
		SubtitleTime time = sub.get_start() - SubtitleTime(0,0,4,0);

		start_position = time.hours*3600 + time.mins*60 + time.secs;

		if(start_position < 0)
			start_position = 0;
	}

	// command pipe...
	Glib::ustring cmd;

	// load the config or use the default command
	if(!Config::getInstance().get_value_string("external-video-player", "command", cmd))
	{
		Glib::ustring default_cmd = "mplayer \"#video_file\" -sub \"#subtitle_file\" -ss #seconds -osdlevel 2";
		Config::getInstance().set_value_string("external-video-player", "command", default_cmd);
		cmd = default_cmd;
	}
	// create the command
	{
		Glib::ustring video_uri = uri;
		Glib::ustring video_file = Glib::filename_from_uri(video_uri);

		Glib::ustring seconds = to_string(start_position);
			
		Glib::ustring subtitle_uri = Glib::filename_to_uri(subtitle_file);
			
		find_and_replace(cmd, "#video_file", video_file);
		find_and_replace(cmd, "#video_uri", video_uri);
		find_and_replace(cmd, "#subtitle_file", subtitle_file);
		find_and_replace(cmd, "#subtitle_uri", subtitle_uri);
		find_and_replace(cmd, "#seconds", seconds);
	}

	std::cout << "COMMAND: " << cmd << std::endl;

	Glib::spawn_command_line_async(cmd);
	return true;
}
