#include "InsertSubtitle.h"


/*
 *
 */
bool InsertSubtitle::execute(Document &doc, POSITION pos)
{
	if(pos == BEFORE)
		return insert_before(doc);

	return insert_after(doc);
}

/*
 *
 */
bool InsertSubtitle::insert_before(Document &doc)
{
	SubtitleViewPtr view = doc.get_subtitle_view();
	SubtitleModelPtr model = doc.get_subtitle_model();

	Gtk::TreeIter select = view->getSelected();
	
	if(!select)
	{
		// si le model est vide alors on crée un par defaut
		if(model->getSize() == 0)
		{
			Gtk::TreeIter it = model->append();
			view->select_and_set_cursor(it);
			return true;
		}
		else
			return false;
	}
	int gap_between_subtitle;
	Config::getInstance().get_value_int("timing", "min-gap-between-subtitle", gap_between_subtitle, 100);

	SubtitleTime gap(gap_between_subtitle);

	SubtitleTime time_start;
	SubtitleModifier prev(model->find_previous(select));

	if(prev)
	{
		time_start = prev.get_end() + gap;
	}

	// on recupere le temps de depart pour s'en servir comme temps de fin
	SubtitleTime time_end = SubtitleModifier(*select).get_start() - gap;

	Gtk::TreeIter new_iter = model->insertBefore(select);

	SubtitleModifier ni(new_iter);
	ni.set_start_and_end(time_start, time_end);

	view->select_and_set_cursor(new_iter);
	
	return true;
}

/*
 *
 */
bool InsertSubtitle::insert_after(Document &doc)
{
	SubtitleViewPtr view = doc.get_subtitle_view();
	SubtitleModelPtr model = doc.get_subtitle_model();

	Gtk::TreeIter select = view->getSelected();
	
	if(!select)
	{
		// si le model est vide alors on crée un par defaut
		if(model->getSize() == 0)
		{
			Gtk::TreeIter it = model->append();
			view->select_and_set_cursor(it);
			return true;
		}
		else
			return false;
	}

	int gap_between_subtitle;
	Config::getInstance().get_value_int("timing", "min-gap-between-subtitle", gap_between_subtitle, 100);

	SubtitleTime time_start, time_end;

	time_start = SubtitleModifier(select).get_end() + SubtitleTime(0,0,0, gap_between_subtitle);

	SubtitleModifier next(model->find_next(select));

	if(next)
	{
		time_end = next.get_start() - SubtitleTime(0, 0, 0, gap_between_subtitle);
	}
	else	// s'il n'y a pas de suivant on ajoute 2 secs par defaut
		time_end = time_start + SubtitleTime(0,0,2,0);

	Gtk::TreeIter new_iter = model->insertAfter(select);

	SubtitleModifier ni(new_iter);
	ni.set_start_and_end(time_start, time_end);

	view->select_and_set_cursor(new_iter);
	
	return true;
}
