#ifndef _CheckErrorsUI_h
#define _CheckErrorsUI_h
/*
 * Overlapping (chevauchement) : Une erreur est détectée lorsque deux sous-titres se chevauchent.
 * Too short display time (temps trop court) : Une erreur est détectée lorsque le nombre de caractères par seconde est strictement supérieur à une valeur définie par l'utilisateur.
 * Too long display time (temps trop long) : Une erreur est détectée lorsque le nombre de caractères par seconde est strictement inférieur à une valeur définie par l'utilisateur.
 * Too long line (ligne trop longue) : Une erreur est détectée lorsque la longueur d'une ligne de sous-titre est strictement supérieur à une valeur spécifiée par l'utilisateur.
 * Transition (respect le temps entre chaque sous-titre)
*/

#include "Document.h"

// TODO: ajouter des fonctions qui corrige certain probleme automatiquement
/*
 *
 */
class CheckErrors
{
public:
	CheckErrors();
	~CheckErrors();

	/*
	 *	chevauchement
	 *	Une erreur est détectée lorsque deux sous-titres se chevauchent.
	 */
	bool check_overlapping(const SubtitleModifier &sub, Glib::ustring &msg);

	/*
	 *	temps trop court
	 *	Une erreur est détectée lorsque le nombre de caractères par seconde
	 *	est strictement supérieur à une valeur définie par l'utilisateur.
	 */
	bool check_short_display_time(const SubtitleModifier &sub, Glib::ustring &msg);
	
	/*
	 *	temps trop long
	 *	Une erreur est détectée lorsque le nombre de caractères par seconde
	 *	est strictement inférieur à une valeur définie par l'utilisateur.
	 */
	bool check_long_display_time(const SubtitleModifier &sub, Glib::ustring &msg);

	/*
	 *	transition
	 *	Respect du temps entre chaque s-t
	 */
	bool check_minimum_gap_between_subtitle(const SubtitleModifier &sub, Glib::ustring &msg);

	/*
	 *	affichage trop court d'un sous-titre
	 */
	bool check_minimum_display_subtitle(const SubtitleModifier &sub, Glib::ustring &msg);

	/*
	 *	ligne trop longue
	 *	Une erreur est détectée lorsque la longueur d'une ligne de sous-titre
	 *	est strictement supérieur à une valeur spécifiée par l'utilisateur.
	 */
	bool check_long_line_text(const SubtitleModifier &sub, Glib::ustring &msg);
	bool check_long_line_translation(const SubtitleModifier &sub, Glib::ustring &msg);

	/*
	 *	trop de ligne
	 */
	bool check_max_line_text(const SubtitleModifier &sub, Glib::ustring &msg);
	bool check_max_line_translation(const SubtitleModifier &sub, Glib::ustring &msg);
protected:

	void load_config();

	/*
	 *	calcul le nombre de caractères par seconde
	 */
	int calcul_cps(const SubtitleModifier &sub);

protected:
	int m_minCPS; // minimum caractères par second
	int m_maxCPS; // maximun caractères par seconde
	int m_maxCPL; // maximun caractères par ligne
	int m_minDisplay; // affichage minimum du s-t en ms
	int m_minGBS; // gab between subtitle
	int m_maxLine; // maximum de ligne par sous-titre
};


void createDialogCheckErrors();

#endif//_CheckErrorsUI_h

