/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include <gtkmm.h>
#include <libglademm/xml.h>
#include "utility.h"
#include "DocumentSystem.h"
#include "MoveSubtitlesUI.h"
#include "actions/MoveSubtitles.h"

/*
 *
 */
class DialogMoveSubtitle : public Gtk::Dialog
{
public:
	DialogMoveSubtitle(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::Dialog(cobject)
	{
		refGlade->get_widget_derived("spin-current-time", m_spinCurrentTime);
		refGlade->get_widget_derived("spin-new-time", m_spinNewTime);


	}

	void execute()
	{
		Document *doc = DocumentSystem::getInstance().getCurrentDocument();

		SubtitleModifier sub( doc->get_subtitle_view()->getSelected() );

		if(!sub)
		{
			//please select subtitle
			return;
		}
		
		SubtitleTime time = sub.get_start();

		m_spinCurrentTime->set_value(time.totalmsecs);
		m_spinCurrentTime->set_range(time.totalmsecs, time.totalmsecs);

		m_spinNewTime->set_value(time.totalmsecs);
		m_spinNewTime->grab_focus();

		show();

		if(run() == Gtk::RESPONSE_OK)
		{
			SubtitleTime newtime( (long int) (m_spinNewTime->get_value() - m_spinCurrentTime->get_value()));
			
			MoveSubtitles action;
			action.execute(*doc, newtime.totalmsecs, true, true, false);
		}

		hide();
	}
protected:
	SpinButtonTiming*	m_spinCurrentTime;
	SpinButtonTiming*	m_spinNewTime;
};




/*
 *
 */
void MoveSubtitlesUI::execute()
{
	DialogMoveSubtitle *dialog = NULL;

	Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-move-subtitles.glade"));

	refXml->get_widget_derived("dialog-move-subtitles", dialog);

	dialog->execute();

	delete dialog;
}
